;BlitzBoy
;
;Created by Jan Ebbe on 05.01.2012


Include "cpu.bb"
Include "gpu.bb"
Include "keypad.bb"
Include "timer.bb"
Include "memory.bb"
Include "gui.bb"
Include "sound.bb"
Include "settings.bb"

gui_init()
sound_init()
settings_load()

Repeat
	WaitEvent()
	
	Select EventID()
		Case $101
			If memory_rom_size > 0 Then keypad_down(EventData())
			
		Case $102
			If memory_rom_size > 0 Then keypad_up(EventData())
			
		Case $201
			If memory_rom_size > 0 Then gui_mouse_down(EventData())
			
		Case $202
			If memory_rom_size > 0 Then gui_mouse_up(EventData())
	
		Case $401
			If EventSource() = gui_controls_button_cancel Then ;Controls Cancel
				HideGadget gui_controls_window
				
			Else If EventSource() = gui_controls_button_default Then ;Controls Default
				settings_default()
				gui_controls_update()
				
			Else If EventSource() = gui_controls_button_save Then ;Controls Save
				gui_controls_save()
				settings_write()
				
				HideGadget gui_controls_window
			Else If EventSource() = gui_sound_checkbox_enable Then ;Sound Enable
				sound_master_enabled = ButtonState(gui_sound_checkbox_enable)
				If sound_master_enabled = 1 Then
					BASS_Start()
				Else
					BASS_Pause()
				End If
				
			Else If EventSource() = gui_sound_slider_volume Then ;Volume Slider
				sound_master_volume = SliderValue(gui_sound_slider_volume)
				BASS_SetVolume(sound_master_volume)
				
			End If
		
		Case $803
			If EventSource() = gui_window Then ;Main Window Close
				sound_stop()
				End
				
			Else If EventSource() = gui_controls_window Then ;Controls Window Close
				HideGadget gui_controls_window
				
			Else If EventSource() = gui_sound_window Then ;Sound Window Close
				settings_write()
				HideGadget gui_sound_window
				
			End If
			
		Case $1001
			Select EventData()
				Case 11 ;Open
					file$ = RequestFile("Select ROM File", "gb", 0)
					If Not file = "" Then
						EnableMenu gui_menu_restart
						UpdateWindowMenu gui_window
						
						blitzboy_reset()
						blitzboy_power_up()
						memory_load_rom (file$)
					End If
				
				Case 12 ;Restart
					file$ = memory_rom_path$
					blitzboy_reset()
					blitzboy_power_up()
					memory_load_rom (file$)
				
				Case 13 ;Exit
					sound_stop()
					End
					
				Case 21 ;Controls
					gui_controls_update()
					ShowGadget gui_controls_window
					
				Case 22 ;Sound
					gui_sound_update()
					ShowGadget gui_sound_window
					
			End Select
		
		Case $2001
			If memory_rom_size > 0 Then keypad_all_up()
		
		Case $4001
			If memory_rom_size > 0 Then
				blitzboy_frame()
			Else
				gui_animation()
			End If
	End Select
Forever


Function blitzboy_reset()
	cpu_reset()
	memory_reset()
	gpu_reset()
	keypad_reset()
	timer_reset()
	sound_reset()
End Function

Function blitzboy_frame()
	Repeat
		If cpu_stop = 0 Then
			cpu_exec()
			counter = counter + cpu_m
			gpu_step()
			timer_step()
			sound_step()
		Else
			counter = counter + 1 ;NOP
		End If
	Until (counter >= 17556)
End Function

Function blitzboy_power_up()
	cpu_a = $01
	cpu_f = $B0
	cpu_b = $00
	cpu_c = $13
	cpu_d = $00
	cpu_e = $D8
	cpu_h = $01
	cpu_l = $4D
	cpu_sp = $FFFE
	cpu_pc = $100
	
	memory_rom_bank = 1
	memory_write_byte($FF05,$00)
	memory_write_byte($FF06,$00)
	memory_write_byte($FF07,$00)
	memory_write_byte($FF10,$80)
	memory_write_byte($FF11,$BF)
	memory_write_byte($FF12,$F3)
	memory_write_byte($FF14,$BF)
	memory_write_byte($FF16,$3F)
	memory_write_byte($FF17,$00)
	memory_write_byte($FF19,$BF)
	memory_write_byte($FF1A,$7F)
	memory_write_byte($FF1B,$FF)
	memory_write_byte($FF1C,$9F)
	memory_write_byte($FF1E,$BF)
	memory_write_byte($FF20,$FF)
	memory_write_byte($FF21,$00)
	memory_write_byte($FF22,$00)
	memory_write_byte($FF23,$BF)
	memory_write_byte($FF24,$77)
	memory_write_byte($FF25,$F3)
	memory_write_byte($FF26,$F1)
	memory_write_byte($FF40,$91)
	memory_write_byte($FF42,$00)
	memory_write_byte($FF43,$00)
	memory_write_byte($FF45,$00)
	memory_write_byte($FF47,$FC)
	memory_write_byte($FF48,$FF)
	memory_write_byte($FF49,$FF)
	memory_write_byte($FF4A,$00)
	memory_write_byte($FF4B,$00)
	memory_write_byte($FFFF,$00)
	
	gpu_mode = 2
	For n=0 To 39 ;Hides all sprites
		gpu_sprites(n,0) = -16
		gpu_sprites(n,1) = -8
	Next
	
	keypad_rows (0) = $0F
	keypad_rows (1) = $0F
End Function