;cpu.bb
;BlitzBoy
;
;Created by Jan Ebbe on 05.01.2012


;8Bit Registers
Global cpu_a
Global cpu_b
Global cpu_c
Global cpu_d
Global cpu_e
Global cpu_h
Global cpu_l
Global cpu_f

;Stack Pointer
Global cpu_sp

;Program Counter
Global cpu_pc

;Interrupts Enable
Global cpu_ime

;Clock
Global cpu_m

;Halt
Global cpu_halt
Global cpu_stop


Function cpu_reset()
	cpu_a = 0
	cpu_b = 0
	cpu_c = 0
	cpu_d = 0
	cpu_e = 0
	cpu_h = 0
	cpu_l = 0
	cpu_f = 0
	cpu_sp = 0
	cpu_pc = 0
	cpu_ime = 1
	cpu_m = 0
	cpu_halt = 0
	cpu_stop = 0
End Function

Function cpu_exec()
	;Interrupts
	If cpu_ime = 1 And (memory_ie And $01) > 0 And (memory_if And $01) > 0 Then ;VBlank
		cpu_ime = 0
		cpu_halt = 0
		memory_if = memory_if - $01
		cpu_rst($40)
	Else If cpu_ime = 1 And (memory_ie And $02) > 0 And (memory_if And $02) > 0 Then ;LCDC Status
		cpu_halt = 0
		cpu_ime = 0
		memory_if = memory_if - $02
		cpu_rst($48)
	Else If cpu_ime = 1 And (memory_ie And $04) > 0 And (memory_if And $04) > 0 Then ;Timer Overflow
		cpu_halt = 0
		cpu_ime = 0
		memory_if = memory_if - $04
		cpu_rst($50)
	Else If cpu_ime = 1 And (memory_ie And $10) > 0 And (memory_if And $10) > 0 Then ;Keypad
		cpu_halt = 0
		cpu_ime = 0
		memory_if = memory_if - $10
		cpu_rst($60)
	Else If cpu_halt = 0 Then
		opcode = memory_read_byte(cpu_pc) ;Read Opcode
		cpu_pc = cpu_pc +1 ;Increase Program Counter
		cpu_opcode(opcode)
	Else If cpu_halt = 1 And cpu_ime = 0 Then
		Notify "Cpu halted and interupts disabled!"
	End If
End Function

Function cpu_opcode(opcode)
	Select opcode
	
		;8Bit LD
		Case $7F ;LD A,A
			cpu_m = 1
		Case $78 ;LD A,B
			cpu_m = 1
			cpu_a = cpu_b
		Case $79 ;LD A,C
			cpu_m = 1
			cpu_a = cpu_c
		Case $7A ;LD A,D
			cpu_m = 1
			cpu_a = cpu_d
		Case $7B ;LD A,E
			cpu_m = 1
			cpu_a = cpu_e
		Case $7C ;LD A,H
			cpu_m = 1
			cpu_a = cpu_h
		Case $7D ;LD A,L
			cpu_m = 1
			cpu_a = cpu_l
		Case $47 ;LD B,A
			cpu_m = 1
			cpu_b = cpu_a
		Case $40 ;LD B,B
			cpu_m = 1
		Case $41 ;LD B,C
			cpu_m = 1
			cpu_b = cpu_c
		Case $42 ;LD B,D
			cpu_m = 1
			cpu_b = cpu_d
		Case $43 ;LD B,E
			cpu_m = 1
			cpu_b = cpu_e
		Case $44 ;LD B,H
			cpu_m = 1
			cpu_b = cpu_h
		Case $45 ;LD B,L
			cpu_m = 1
			cpu_b = cpu_l
		Case $4F ;LD C,A
			cpu_m = 1
			cpu_c = cpu_a
		Case $48 ;LD C,B
			cpu_m = 1
			cpu_c = cpu_b
		Case $49 ;LD C,C
			cpu_m = 1
		Case $4A ;LD C,D
			cpu_m = 1
			cpu_c = cpu_d
		Case $4B ;LD C,E
			cpu_m = 1
			cpu_c = cpu_e
		Case $4C ;LD C,H
			cpu_m = 1
			cpu_c = cpu_h
		Case $4D ;LD C,L
			cpu_m = 1
			cpu_c = cpu_l
		Case $57 ;LD D,A
			cpu_m = 1
			cpu_d = cpu_a
		Case $50 ;LD D,B
			cpu_m = 1
			cpu_d = cpu_b
		Case $51 ;LD D,C
			cpu_m = 1
			cpu_d = cpu_c
		Case $52 ;LD D,D
			cpu_m = 1
		Case $53 ;LD D,E
			cpu_m = 1
			cpu_d = cpu_e
		Case $54 ;LD D,H
			cpu_m = 1
			cpu_d = cpu_h
		Case $55 ;LD D,L
			cpu_m = 1
			cpu_d = cpu_l
		Case $5F ;LD E,A
			cpu_m = 1
			cpu_e = cpu_a
		Case $58 ;LD E,B
			cpu_m = 1
			cpu_e = cpu_b
		Case $59 ;LD E,C
			cpu_m = 1
			cpu_e = cpu_c
		Case $5A ;LD E,D
			cpu_m = 1
			cpu_e = cpu_d
		Case $5B ;LD E,E
			cpu_m = 1
		Case $5C ;LD E,H
			cpu_m = 1
			cpu_e = cpu_h
		Case $5D ;LD E,L
			cpu_m = 1
			cpu_e = cpu_l
		Case $67 ;LD H,A
			cpu_m = 1
			cpu_h = cpu_a
		Case $60 ;LD H,B
			cpu_m = 1
			cpu_h = cpu_b
		Case $61 ;LD H,C
			cpu_m = 1
			cpu_h = cpu_c
		Case $62 ;LD H,D
			cpu_m = 1
			cpu_h = cpu_d
		Case $63 ;LD H,E
			cpu_m = 1
			cpu_h = cpu_e
		Case $64 ;LD H,H
			cpu_m = 1
		Case $65 ;LD H,L
			cpu_m = 1
			cpu_h = cpu_l
		Case $6F ;LD L,A
			cpu_m = 1
			cpu_l = cpu_a
		Case $68 ;LD L,B
			cpu_m = 1
			cpu_l = cpu_b
		Case $69 ;LD L,C
			cpu_m = 1
			cpu_l = cpu_c
		Case $6A ;LD L,D
			cpu_m = 1
			cpu_l = cpu_d
		Case $6B ;LD L,E
			cpu_m = 1
			cpu_l = cpu_e
		Case $6C ;LD L,H
			cpu_m = 1
			cpu_l = cpu_h
		Case $6D ;LD L,L
			cpu_m = 1
		Case $3E ;LD A,XX
			cpu_m = 2
			cpu_a = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $06 ;LD B,XX
			cpu_m = 2
			cpu_b = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $0E ;LD C,XX
			cpu_m = 2
			cpu_c = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $16 ;LD D,XX
			cpu_m = 2
			cpu_d = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $1E ;LD E,XX
			cpu_m = 2
			cpu_e = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $26 ;LD H,XX
			cpu_m = 2
			cpu_h = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $2E ;LD L,XX
			cpu_m = 2
			cpu_l = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
		Case $7E ;LD A,(HL)
			cpu_m = 2
			cpu_a = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $46 ;LD B,(HL)
			cpu_m = 2
			cpu_b = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $4E ;LD C,(HL)
			cpu_m = 2
			cpu_c = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $56 ;LD D,(HL)
			cpu_m = 2
			cpu_d = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $5E ;LD E,(HL)
			cpu_m = 2
			cpu_e = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $66 ;LD H,(HL)
			cpu_m = 2
			cpu_h = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $6E ;LD L,(HL)
			cpu_m = 2
			cpu_l = memory_read_byte((cpu_h Shl 8) + cpu_l)
		Case $77 ;LD (HL),A
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_a)
		Case $70 ;LD (HL),B
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_b)
		Case $71 ;LD (HL),C
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_c)
		Case $72 ;LD (HL),D
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_d)
		Case $73 ;LD (HL),E
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_e)
		Case $74 ;LD (HL),H
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_h)
		Case $75 ;LD (HL),L
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_l)
		Case $36 ;LD (HL),XX
			cpu_m = 3
			memory_write_byte((cpu_h Shl 8) + cpu_l,memory_read_byte(cpu_pc))
			cpu_pc = cpu_pc +1
		Case $0A ;LD A,(BC)
			cpu_m = 2
			cpu_a = memory_read_byte((cpu_b Shl 8) + cpu_c)
		Case $1A ;LD A,(DE)
			cpu_m = 2
			cpu_a = memory_read_byte((cpu_d Shl 8) + cpu_e)
		Case $F2 ;LD A,($FF00+C)
			cpu_m = 2
			cpu_a = memory_read_byte(cpu_c + $FF00)
		Case $E2 ;LD ($FF00+C),A
			cpu_m = 2
			memory_write_byte(cpu_c + $FF00,cpu_a)
		Case $F0 ;LD A,($FF00+XX)
			cpu_m = 3
			cpu_a = memory_read_byte(memory_read_byte(cpu_pc) + $FF00)
			cpu_pc = cpu_pc +1
		Case $E0 ;LD ($FF00+XX),A
			cpu_m = 3
			memory_write_byte(memory_read_byte(cpu_pc) + $FF00,cpu_a)
			cpu_pc = cpu_pc +1
		Case $FA ;LD A,(XXYY)
			cpu_m = 4
			cpu_a = memory_read_byte(memory_read_word(cpu_pc))
			cpu_pc = cpu_pc +2
		Case $EA ;LD (XXYY),A
			cpu_m = 4
			memory_write_byte(memory_read_word(cpu_pc),cpu_a)
			cpu_pc = cpu_pc +2
		Case $2A ;LD A,(HLI)
			cpu_m = 2
			cpu_a = memory_read_byte((cpu_h Shl 8) + cpu_l)
			cpu_l = (cpu_l +1) And $FF
			If cpu_l = 0 Then cpu_h = (cpu_h +1) And $FF
		Case $3A ;LD A,(HLD)
			cpu_m = 2
			cpu_a = memory_read_byte((cpu_h Shl 8) + cpu_l)
			cpu_l = (cpu_l -1) And $FF
			If cpu_l = $FF Then cpu_h = (cpu_h -1) And $FF
		Case $02 ;LD (BC),A
			cpu_m = 2
			memory_write_byte((cpu_b Shl 8) + cpu_c,cpu_a)
		Case $12 ;LD (DE),A
			cpu_m = 2
			memory_write_byte((cpu_d Shl 8) + cpu_e,cpu_a)
		Case $22 ;LD (HLI),A
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_a)
			cpu_l = (cpu_l +1) And $FF
			If cpu_l = 0 Then cpu_h = (cpu_h +1) And $FF
		Case $32 ;LD (HLD),A
			cpu_m = 2
			memory_write_byte((cpu_h Shl 8) + cpu_l,cpu_a)
			cpu_l = (cpu_l -1) And $FF
			If cpu_l = $FF Then cpu_h = (cpu_h -1) And $FF
			
			
		;16Bit LD
		Case $01 ;LD BC,XXYY
			cpu_m = 3
			cpu_c = memory_read_byte(cpu_pc)
			cpu_b = memory_read_byte(cpu_pc+1)
			cpu_pc = cpu_pc +2
		Case $11 ;LD DE,XXYY
			cpu_m = 3
			cpu_e = memory_read_byte(cpu_pc)
			cpu_d = memory_read_byte(cpu_pc+1)
			cpu_pc = cpu_pc +2
		Case $21 ;LD HL,XXYY
			cpu_m = 3
			cpu_l = memory_read_byte(cpu_pc)
			cpu_h = memory_read_byte(cpu_pc+1)
			cpu_pc = cpu_pc +2
		Case $31 ;LD SP,XXYY
			cpu_m = 3
			cpu_sp = memory_read_word(cpu_pc)
			cpu_pc = cpu_pc +2
		Case $F9 ;LD SP,HL
			cpu_m = 3
			cpu_sp = (cpu_h Shl 8) + cpu_l
		Case $F8 ;LD HL,SP+XX
			cpu_m = 3
			value = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
			value2 = (value + cpu_sp) And $FFFF
			cpu_h = (value2 Shr 8) And $FF
			cpu_l = value2 And $FF
			value = cpu_sp Xor value Xor value2
			cpu_f = 0
			If (value And $100) > 0 Then cpu_f = cpu_f Or $10
			If (value And $10) > 0 Then cpu_f = cpu_f Or $20
			
			
		Case $08 ;LD (XXYY),SP
			cpu_m = 3
			memory_write_word(memory_read_word(cpu_pc),cpu_sp)
			cpu_pc = cpu_pc +2
			
			
		;16Bit PUSH
		Case $F5 ;PUSH AF
			cpu_m = 3
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_a)
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_f)	
		Case $C5 ;PUSH BC
			cpu_m = 3
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_b)
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_c)
		Case $D5 ;PUSH DE
			cpu_m = 3
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_d)
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_e)
		Case $E5 ;PUSH HL
			cpu_m = 3
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_h)
			cpu_sp = cpu_sp -1
			memory_write_byte(cpu_sp, cpu_l)
			
			
		;16Bit POP
		Case $F1 ;POP AF
			cpu_m = 3
			cpu_f = (memory_read_byte(cpu_sp) And $F0)
			cpu_sp = cpu_sp +1
			cpu_a = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
		Case $C1 ;POP BC
			cpu_m = 3
			cpu_c = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
			cpu_b = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
		Case $D1 ;POP DE
			cpu_m = 3
			cpu_e = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
			cpu_d = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
		Case $E1 ;POP HL
			cpu_m = 3
			cpu_l = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
			cpu_h = memory_read_byte(cpu_sp)
			cpu_sp = cpu_sp +1
			
			
		;8Bit ADD
		Case $87 ;ADD A,A
			cpu_m = 1
			value = cpu_a + cpu_a
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_a And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $80 ;ADD A,B
			cpu_m = 1
			value = cpu_a + cpu_b
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_b And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $81 ;ADD A,C
			cpu_m = 1
			value = cpu_a + cpu_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_c And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $82 ;ADD A,D
			cpu_m = 1
			value = cpu_a + cpu_d
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_d And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $83 ;ADD A,E
			cpu_m = 1
			value = cpu_a + cpu_e
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_e And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $84 ;ADD A,H
			cpu_m = 1
			value = cpu_a + cpu_h
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_h And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $85 ;ADD A,L
			cpu_m = 1
			value = cpu_a + cpu_l
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_l And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $86 ;ADD A,(HL)
			cpu_m = 2
			add = memory_read_byte((cpu_h Shl 8) + cpu_l)
			value = cpu_a + add
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (add And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $C6 ;ADD A,XX
			cpu_m = 2
			add = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			value = cpu_a + add
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (add And $0F)) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF


		;8Bit ADC
		Case $8F ;ADC A,A
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_a + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_a And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $88 ;ADC A,B
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_b + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_b And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $89 ;ADC A,C
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_c + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_c And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $8A ;ADC A,D
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_d + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_d And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $8B ;ADC A,E
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_e + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_e And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $8C ;ADC A,H
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_h + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_h And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $8D ;ADC A,L
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a + cpu_l + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (cpu_l And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $8E ;ADC A,(HL)
			cpu_m = 2
			If (cpu_f And $10) > 0 Then flag_c = 1
			add = memory_read_byte((cpu_h Shl 8) + cpu_l)
			value = cpu_a + add + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (add And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $CE ;ADC A,XX
			cpu_m = 2
			If (cpu_f And $10) > 0 Then flag_c = 1
			add = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			value = cpu_a + add + flag_c
			cpu_f = 0
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If (value - $FF) > 0 Then cpu_f = cpu_f Or $10
			If (((cpu_a And $0F) + (add And $0F) + flag_c) And $10) = $10 Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF


		;8Bit SUB
		Case $97 ;SUB A
			cpu_m = 1
			value = cpu_a - cpu_a
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_a And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $90 ;SUB B
			cpu_m = 1
			value = cpu_a - cpu_b
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_b And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $91 ;SUB C
			cpu_m = 1
			value = cpu_a - cpu_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_c And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $92 ;SUB D
			cpu_m = 1
			value = cpu_a - cpu_d
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_d And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $93 ;SUB E
			cpu_m = 1
			value = cpu_a - cpu_e
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_e And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $94 ;SUB H
			cpu_m = 1
			value = cpu_a - cpu_h
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_h And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $95 ;SUB L
			cpu_m = 1
			value = cpu_a - cpu_l
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_l And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $96 ;SUB (HL)
			cpu_m = 2
			sub = memory_read_byte((cpu_h Shl 8) + cpu_l)
			value = cpu_a - sub
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (sub And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $D6 ;SUB XX
			cpu_m = 2
			sub = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			value = cpu_a - sub
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (sub And $0F) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
			

		;8Bit SBC
		Case $9F ;SBC A,A
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_a - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_a And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $98 ;SBC A,B
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_b - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_b And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $99 ;SBC A,C
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_c - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_c And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $9A ;SBC A,D
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_d - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_d And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $9B ;SBC A,E
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_e - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_e And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $9C ;SBC A,H
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_h - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_h And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $9D ;SBC A,L
			cpu_m = 1
			If (cpu_f And $10) > 0 Then flag_c = 1
			value = cpu_a - cpu_l - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((cpu_l And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $9E ;SBC A,(HL)
			cpu_m = 2
			If (cpu_f And $10) > 0 Then flag_c = 1
			sub = memory_read_byte((cpu_h Shl 8) + cpu_l)
			value = cpu_a - sub - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((sub And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
		Case $DE ;SBC A,XX
			cpu_m = 2
			If (cpu_f And $10) > 0 Then flag_c = 1
			sub = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			value = cpu_a - sub - flag_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < ((sub And $0F) + flag_c) Then cpu_f = cpu_f Or $20
			cpu_a = value And $FF
			

		;8Bit AND
		Case $A7 ;AND A
			cpu_m = 1
			cpu_a = cpu_a And cpu_a
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A0 ;AND B
			cpu_m = 1
			cpu_a = cpu_a And cpu_b
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A1 ;AND C
			cpu_m = 1
			cpu_a = cpu_a And cpu_c
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A2 ;AND D
			cpu_m = 1
			cpu_a = cpu_a And cpu_d
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A3 ;AND E
			cpu_m = 1
			cpu_a = cpu_a And cpu_e
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A4 ;AND H
			cpu_m = 1
			cpu_a = cpu_a And cpu_h
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A5 ;AND L
			cpu_m = 1
			cpu_a = cpu_a And cpu_l
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A6 ;AND (HL)
			cpu_m = 2
			cpu_a = cpu_a And memory_read_byte((cpu_h Shl 8) + cpu_l)
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $E6 ;AND XX
			cpu_m = 2
			cpu_a = cpu_a And memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			cpu_f = $20
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
			
			
		;8Bit OR
		Case $B7 ;OR A
			cpu_m = 1
			cpu_a = cpu_a Or cpu_a
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B0 ;OR B
			cpu_m = 1
			cpu_a = cpu_a Or cpu_b
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B1 ;OR C
			cpu_m = 1
			cpu_a = cpu_a Or cpu_c
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B2 ;OR D
			cpu_m = 1
			cpu_a = cpu_a Or cpu_d
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B3 ;OR E
			cpu_m = 1
			cpu_a = cpu_a Or cpu_e
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B4 ;OR H
			cpu_m = 1
			cpu_a = cpu_a Or cpu_h
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B5 ;OR L
			cpu_m = 1
			cpu_a = cpu_a Or cpu_l
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $B6 ;OR (HL)
			cpu_m = 2
			cpu_a = cpu_a Or memory_read_byte((cpu_h Shl 8) + cpu_l)
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $F6 ;OR XX
			cpu_m = 2
			cpu_a = cpu_a Or memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
			
			
		;8Bit XOR
		Case $AF ;XOR A
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_a
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A8 ;XOR B
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_b
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $A9 ;XOR C
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_c
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $AA ;XOR D
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_d
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $AB ;XOR E
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_e
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $AC ;XOR H
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_h
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $AD ;XOR L
			cpu_m = 1
			cpu_a = cpu_a Xor cpu_l
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $AE ;XOR (HL)
			cpu_m = 2
			cpu_a = cpu_a Xor memory_read_byte((cpu_h Shl 8) + cpu_l)
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
		Case $EE ;XOR XX
			cpu_m = 2
			cpu_a = cpu_a Xor memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			cpu_f = 0
			If cpu_a = 0 Then cpu_f = cpu_f Or $80
			
			
		;8Bit CP
		Case $BF ;CP A
			cpu_m = 1
			value = cpu_a - cpu_a
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_a And $0F) Then cpu_f = cpu_f Or $20
		Case $B8 ;CP B
			cpu_m = 1
			value = cpu_a - cpu_b
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_b And $0F) Then cpu_f = cpu_f Or $20
		Case $B9 ;CP C
			cpu_m = 1
			value = cpu_a - cpu_c
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_c And $0F) Then cpu_f = cpu_f Or $20
		Case $BA ;CP D
			cpu_m = 1
			value = cpu_a - cpu_d
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_d And $0F) Then cpu_f = cpu_f Or $20
		Case $BB ;CP E
			cpu_m = 1
			value = cpu_a - cpu_e
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_e And $0F) Then cpu_f = cpu_f Or $20
		Case $BC ;CP H
			cpu_m = 1
			value = cpu_a - cpu_h
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_h And $0F) Then cpu_f = cpu_f Or $20
		Case $BD ;CP L
			cpu_m = 1
			value = cpu_a - cpu_l
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (cpu_l And $0F) Then cpu_f = cpu_f Or $20
		Case $BE ;CP (HL)
			cpu_m = 2
			sub = memory_read_byte((cpu_h Shl 8) + cpu_l)
			value = cpu_a - sub
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (sub And $0F) Then cpu_f = cpu_f Or $20
		Case $FE ;CP XX
			cpu_m = 2
			sub = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			value = cpu_a - sub
			cpu_f = $40
			If (value And $FF) = 0 Then cpu_f = cpu_f Or $80
			If value<0 Then cpu_f = cpu_f Or $10
			If (cpu_a And $0F) < (sub And $0F) Then cpu_f = cpu_f Or $20
			
		
		;8Bit INC
		Case $3C ;INC A
			cpu_m = 1
			cpu_a = (cpu_a + 1) And $FF
			If cpu_a = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_a And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $04 ;INC B
			cpu_m = 1
			cpu_b = (cpu_b + 1) And $FF
			If cpu_b = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_b And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $0C ;INC C
			cpu_m = 1
			cpu_c = (cpu_c + 1) And $FF
			If cpu_c = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_c And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $14 ;INC D
			cpu_m = 1
			cpu_d = (cpu_d + 1) And $FF
			If cpu_d = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_d And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $1C ;INC E
			cpu_m = 1
			cpu_e = (cpu_e + 1) And $FF
			If cpu_e = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_e And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $24 ;INC H
			cpu_m = 1
			cpu_h = (cpu_h + 1) And $FF
			If cpu_h = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_h And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $2C ;INC L
			cpu_m = 1
			cpu_l = (cpu_l + 1) And $FF
			If cpu_l = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (cpu_l And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
		Case $34 ;INC (HL)
			cpu_m = 3
			value = (memory_read_byte((cpu_h Shl 8) + cpu_l) + 1) And $FF
			memory_write_byte((cpu_h Shl 8) + cpu_l, value)
			If value = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			EndIf
			If (value And $0F) = 0 Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f And $BF
			
			
		;8Bit DEC
		Case $3D ;DEC A
			cpu_m = 1
			cpu_a = (cpu_a - 1) And $FF
			If cpu_a = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_a And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $05 ;DEC B
			cpu_m = 1
			cpu_b = (cpu_b - 1) And $FF
			If cpu_b = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_b And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $0D ;DEC C
			cpu_m = 1
			cpu_c = (cpu_c - 1) And $FF
			If cpu_c = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_c And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $15 ;DEC D
			cpu_m = 1
			cpu_d = (cpu_d - 1) And $FF
			If cpu_d = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_d And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $1D ;DEC E
			cpu_m = 1
			cpu_e = (cpu_e - 1) And $FF
			If cpu_e = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_e And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $25 ;DEC H
			cpu_m = 1
			cpu_h = (cpu_h - 1) And $FF
			If cpu_h = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_h And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $2D ;DEC L
			cpu_m = 1
			cpu_l = (cpu_l - 1) And $FF
			If cpu_l = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (cpu_l And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40
		Case $35 ;DEC (HL)
			cpu_m = 3
			value = (memory_read_byte((cpu_h Shl 8) + cpu_l) - 1) And $FF
			memory_write_byte((cpu_h Shl 8) + cpu_l, value)
			If value = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			If (value And $0F) = $0F Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			EndIf
			cpu_f = cpu_f Or $40

						
		;16Bit ADD
		Case $09 ;ADD HL,BC
			cpu_m = 3
			add1 = ((cpu_h Shl 8) + cpu_l)
			add2 = ((cpu_b Shl 8) + cpu_c)
			value = add1 + add2
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
			If value > $FFFF Then 
				cpu_f = cpu_f Or $10
			Else
				cpu_f = cpu_f And $EF
			End If
			If ($0FFF - (add1 And $0FFF)) < (add2 And $0FFF) Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			End If
			cpu_f = cpu_f And $BF
		Case $19 ;ADD HL,DE
			cpu_m = 3
			add1 = ((cpu_h Shl 8) + cpu_l)
			add2 = ((cpu_d Shl 8) + cpu_e)
			value = add1 + add2
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
			If value > $FFFF Then 
				cpu_f = cpu_f Or $10
			Else
				cpu_f = cpu_f And $EF
			End If
			If ($0FFF - (add1 And $0FFF)) < (add2 And $0FFF) Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			End If
			cpu_f = cpu_f And $BF
		Case $29 ;ADD HL,HL
			cpu_m = 3
			add1 = ((cpu_h Shl 8) + cpu_l)
			add2 = ((cpu_h Shl 8) + cpu_l)
			value = add1 + add2
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
			If value > $FFFF Then 
				cpu_f = cpu_f Or $10
			Else
				cpu_f = cpu_f And $EF
			End If
			If ($0FFF - (add1 And $0FFF)) < (add2 And $0FFF) Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			End If
			cpu_f = cpu_f And $BF
		Case $39 ;ADD HL,SP
			cpu_m = 3
			add1 = ((cpu_h Shl 8) + cpu_l)
			add2 = cpu_sp
			value = add1 + add2
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
			If value > $FFFF Then 
				cpu_f = cpu_f Or $10
			Else
				cpu_f = cpu_f And $EF
			End If
			If ($0FFF - (add1 And $0FFF)) < (add2 And $0FFF) Then
				cpu_f = cpu_f Or $20
			Else
				cpu_f = cpu_f And $DF
			End If
			cpu_f = cpu_f And $BF
		Case $E8 ;ADD SP,XX
			cpu_m = 4
			value2 = memory_read_byte(cpu_pc)
			If value2 > $7F Then value2 = (-((value2 Xor $FF) And $FF)-1); Complement
			cpu_pc = cpu_pc +1
			value = (cpu_sp + value2) And $FFFF
			value2 = cpu_sp Xor value2 Xor value
			cpu_sp = value
			cpu_f = 0
			If (value2 And $100) = $100 Then cpu_f = cpu_f Or $10
			If (value2 And $10) = $10 Then cpu_f = cpu_f Or $20
			
			
		;16Bit INC
		Case $03 ;INC BC
			cpu_m = 1
			value = (((cpu_b Shl 8) + cpu_c) + 1) And $FFFF
			cpu_b = (value Shr 8) And $FF
			cpu_c = value And $FF
		Case $13 ;INC DE
			cpu_m = 1
			value = (((cpu_d Shl 8) + cpu_e) + 1) And $FFFF
			cpu_d = (value Shr 8) And $FF
			cpu_e = value And $FF
		Case $23 ;INC HL
			cpu_m = 1
			value = (((cpu_h Shl 8) + cpu_l) + 1) And $FFFF
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
		Case $33 ;INC SP
			cpu_m = 1
			cpu_sp = (cpu_sp + 1) And $FFFF
			
			
		;16Bit DEC
		Case $0B ;DEC BC
			cpu_m = 1
			value = (((cpu_b Shl 8) + cpu_c) - 1) And $FFFF
			cpu_b = (value Shr 8) And $FF
			cpu_c = value And $FF
		Case $1B ;DEC DE
			cpu_m = 1
			value = (((cpu_d Shl 8) + cpu_e) - 1) And $FFFF
			cpu_d = (value Shr 8) And $FF
			cpu_e = value And $FF
		Case $2B ;DEC HL
			cpu_m = 1
			value = (((cpu_h Shl 8) + cpu_l) - 1) And $FFFF
			cpu_h = (value Shr 8) And $FF
			cpu_l = value And $FF
		Case $3B ;DEC SP
			cpu_m = 1
			cpu_sp = (cpu_sp - 1) And $FFFF
			
			
		;8Bit Rotates, Shifts, Bit Manipulations
		Case $CB
			opcode2 = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc +1
			
			Select opcode2
				Case $37 ;SWAP A
					cpu_m = 2
					value = cpu_a And $0F
					cpu_a = cpu_a Shr 4
					cpu_a = cpu_a Or (value Shl 4)
					cpu_f = 0
					If cpu_a = 0 Then cpu_f = cpu_f Or $80
				Case $30 ;SWAP B
					cpu_m = 2
					value = cpu_b And $0F
					cpu_b = cpu_b Shr 4
					cpu_b = cpu_b Or (value Shl 4)
					cpu_f = 0
					If cpu_b = 0 Then cpu_f = cpu_f Or $80
				Case $31 ;SWAP C
					cpu_m = 2
					value = cpu_c And $0F
					cpu_c = cpu_c Shr 4
					cpu_c = cpu_c Or (value Shl 4)
					cpu_f = 0
					If cpu_c = 0 Then cpu_f = cpu_f Or $80
				Case $32 ;SWAP D
					cpu_m = 2
					value = cpu_d And $0F
					cpu_d = cpu_d Shr 4
					cpu_d = cpu_d Or (value Shl 4)
					cpu_f = 0
					If cpu_d = 0 Then cpu_f = cpu_f Or $80
				Case $33 ;SWAP E
					cpu_m = 2
					value = cpu_e And $0F
					cpu_e = cpu_e Shr 4
					cpu_e = cpu_e Or (value Shl 4)
					cpu_f = 0
					If cpu_e = 0 Then cpu_f = cpu_f Or $80
				Case $34 ;SWAP H
					cpu_m = 2
					value = cpu_h And $0F
					cpu_h = cpu_h Shr 4
					cpu_h = cpu_h Or (value Shl 4)
					cpu_f = 0
					If cpu_h = 0 Then cpu_f = cpu_f Or $80
				Case $35 ;SWAP L
					cpu_m = 2
					value = cpu_l And $0F
					cpu_l = cpu_l Shr 4
					cpu_l = cpu_l Or (value Shl 4)
					cpu_f = 0
					If cpu_l = 0 Then cpu_f = cpu_f Or $80
				Case $36 ;SWAP (HL)
					cpu_m = 4
					value1 = memory_read_byte((cpu_h Shl 8) + cpu_l)
					value2 = value1 And $0F
					value1 = value1 Shr 4
					value1 = value1 Or (value2 Shl 4)
					memory_write_byte((cpu_h Shl 8) + cpu_l, value1)
					cpu_f = 0
					If value1 = 0 Then cpu_f = cpu_f Or $80
					
				Case $07 ;RLC A
					cpu_m = 2
					If (cpu_a And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_a = (((cpu_a Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_a = 0 Then cpu_f = cpu_f Or $80
				Case $00 ;RLC B
					cpu_m = 2
					If (cpu_b And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_b = (((cpu_b Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_b = 0 Then cpu_f = cpu_f Or $80
				Case $01 ;RLC C
					cpu_m = 2
					If (cpu_c And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_c = (((cpu_c Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_c = 0 Then cpu_f = cpu_f Or $80
				Case $02 ;RLC D
					cpu_m = 2
					If (cpu_d And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_d = (((cpu_d Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_d = 0 Then cpu_f = cpu_f Or $80
				Case $03 ;RLC E
					cpu_m = 2
					If (cpu_e And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_e = (((cpu_e Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_e = 0 Then cpu_f = cpu_f Or $80
				Case $04 ;RLC H
					cpu_m = 2
					If (cpu_h And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_h = (((cpu_h Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_h = 0 Then cpu_f = cpu_f Or $80
				Case $05 ;RLC L
					cpu_m = 2
					If (cpu_l And $80) > 0 Then
						add = 1
						carry = $10
					End If
					cpu_l = (((cpu_l Shl 1) + add) And $FF)
					cpu_f = carry
					If cpu_l = 0 Then cpu_f = cpu_f Or $80
				Case $06 ;RLC (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $80) > 0 Then
						add = 1
						carry = $10
					End If
					value = (((value Shl 1) + add) And $FF)
					cpu_f = carry
					If value = 0 Then cpu_f = cpu_f Or $80
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
	
	
				Case $17 ;RL A
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_a And $80) > 10 Then co = $10
					cpu_a = ((cpu_a Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_a = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $10 ;RL B
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_b And $80) > 10 Then co = $10
					cpu_b = ((cpu_b Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_b = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $11 ;RL C
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_c And $80) > 10 Then co = $10
					cpu_c = ((cpu_c Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_c = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $12 ;RL D
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_d And $80) > 10 Then co = $10
					cpu_d = ((cpu_d Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_d = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $13 ;RL E
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_e And $80) > 10 Then co = $10
					cpu_e = ((cpu_e Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_e = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $14 ;RL H
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_h And $80) > 10 Then co = $10
					cpu_h = ((cpu_h Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_h = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co	
				Case $15 ;RL L
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = 1
					If (cpu_l And $80) > 10 Then co = $10
					cpu_l = ((cpu_l Shl 1) + ci) And $FF
					cpu_f = 0
					If cpu_l = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $16 ;RL (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (cpu_f And $10) > 0 Then ci = 1
					If (value And $80) > 10 Then co = $10
					value = ((value Shl 1) + ci) And $FF
					cpu_f = 0
					If value = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
						
						
				Case $0F ;RRC A
					cpu_m = 2
					If (cpu_a And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_a = (((cpu_a Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_a = 0 Then cpu_f = cpu_f Or $80	
				Case $08 ;RRC B
					cpu_m = 2
					If (cpu_b And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_b = (((cpu_b Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_b = 0 Then cpu_f = cpu_f Or $80		
				Case $09 ;RRC C
					cpu_m = 2
					If (cpu_c And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_c = (((cpu_c Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_c = 0 Then cpu_f = cpu_f Or $80		
				Case $0A ;RRC D
					cpu_m = 2
					If (cpu_d And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_d = (((cpu_d Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_d = 0 Then cpu_f = cpu_f Or $80		
				Case $0B ;RRC E
					cpu_m = 2
					If (cpu_e And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_e = (((cpu_e Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_e = 0 Then cpu_f = cpu_f Or $80		
				Case $0C ;RRC H
					cpu_m = 2
					If (cpu_h And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_h = (((cpu_h Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_h = 0 Then cpu_f = cpu_f Or $80		
				Case $0D ;RRC L
					cpu_m = 2
					If (cpu_l And 1) > 0 Then
						add = $80
						carry = $10
					End If
					cpu_l = (((cpu_l Shr 1) + add) And $FF)
					cpu_f = carry
					If cpu_l = 0 Then cpu_f = cpu_f Or $80		
				Case $0E ;RRC (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And 1) > 0 Then
						add = $80
						carry = $10
					End If
					value = (((value Shr 1) + add) And $FF)
					cpu_f = carry
					If value = 0 Then cpu_f = cpu_f Or $80	
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
					
					
				Case $1F ;RR A
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_a And $01) > 0 Then co = $10
					cpu_a = ((cpu_a Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_a = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $18 ;RR B
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_b And $01) > 0 Then co = $10
					cpu_b = ((cpu_b Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_b = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $19 ;RR C
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_c And $01) > 0 Then co = $10
					cpu_c = ((cpu_c Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_c = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $1A ;RR D
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_d And $01) > 0 Then co = $10
					cpu_d = ((cpu_d Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_d = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $1B ;RR E
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_e And $01) > 0 Then co = $10
					cpu_e = ((cpu_e Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_e = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co	
				Case $1C ;RR H
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_h And $01) > 0 Then co = $10
					cpu_h = ((cpu_h Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_h = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co	
				Case $1D ;RR L
					cpu_m = 2
					If (cpu_f And $10) > 0 Then ci = $80
					If (cpu_l And $01) > 0 Then co = $10
					cpu_l = ((cpu_l Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_l = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $1E ;RR (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (cpu_f And $10) > 0 Then ci = $80
					If (value And $01) > 0 Then co = $10
					value = ((value Shr 1) + ci) And $FF
					cpu_f = 0
					If value = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
	
	
				Case $27 ;SLA A
					cpu_m = 2
					cpu_f = 0
					If (cpu_a And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_a = (cpu_a Shl 1) And $FF
					If cpu_a = 0 Then cpu_f = cpu_f Or $80
				Case $20 ;SLA B
					cpu_m = 2
					cpu_f = 0
					If (cpu_b And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_b = (cpu_b Shl 1) And $FF
					If cpu_b = 0 Then cpu_f = cpu_f Or $80
				Case $21 ;SLA C
					cpu_m = 2
					cpu_f = 0
					If (cpu_c And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_c = (cpu_c Shl 1) And $FF
					If cpu_c = 0 Then cpu_f = cpu_f Or $80
				Case $22 ;SLA D
					cpu_m = 2
					cpu_f = 0
					If (cpu_d And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_d = (cpu_d Shl 1) And $FF
					If cpu_d = 0 Then cpu_f = cpu_f Or $80
				Case $23 ;SLA E
					cpu_m = 2
					cpu_f = 0
					If (cpu_e And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_e = (cpu_e Shl 1) And $FF
					If cpu_e = 0 Then cpu_f = cpu_f Or $80
				Case $24 ;SLA H
					cpu_m = 2
					cpu_f = 0
					If (cpu_h And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_h = (cpu_h Shl 1) And $FF
					If cpu_h = 0 Then cpu_f = cpu_f Or $80
				Case $25 ;SLA L
					cpu_m = 2
					cpu_f = 0
					If (cpu_l And $80) > 0 Then cpu_f = cpu_f Or $10
					cpu_l = (cpu_l Shl 1) And $FF
					If cpu_l = 0 Then cpu_f = cpu_f Or $80
				Case $26 ;SLA (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					cpu_f = 0
					If (value And $80) > 0 Then cpu_f = cpu_f Or $10
					value = (value Shl 1) And $FF
					If value = 0 Then cpu_f = cpu_f Or $80
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
					
					
				Case $2F ;SRA A
					cpu_m = 2
					ci = cpu_a And $80
					If (cpu_a And 1) Then co = $10
					cpu_a = ((cpu_a Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_a = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $28 ;SRA B
					cpu_m = 2
					ci = cpu_b And $80
					If (cpu_b And 1) Then co = $10
					cpu_b = ((cpu_b Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_b = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $29 ;SRA C
					cpu_m = 2
					ci = cpu_c And $80
					If (cpu_c And 1) Then co = $10
					cpu_c = ((cpu_c Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_c = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $2A ;SRA D
					cpu_m = 2
					ci = cpu_d And $80
					If (cpu_d And 1) Then co = $10
					cpu_d = ((cpu_d Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_d = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co	
				Case $2B ;SRA E
					cpu_m = 2
					ci = cpu_e And $80
					If (cpu_e And 1) Then co = $10
					cpu_e = ((cpu_e Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_e = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $2C ;SRA H
					cpu_m = 2
					ci = cpu_h And $80
					If (cpu_h And 1) Then co = $10
					cpu_h = ((cpu_h Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_h = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $2D ;SRA L
					cpu_m = 2
					ci = cpu_l And $80
					If (cpu_l And 1) Then co = $10
					cpu_l = ((cpu_l Shr 1) + ci) And $FF
					cpu_f = 0
					If cpu_l = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $2E ;SRA (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					ci = value And $80
					If (value And 1) Then co = $10
					value = ((value Shr 1) + ci) And $FF
					cpu_f = 0
					If value = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
					
				Case $3F ;SRL A
					cpu_m = 2
					If (cpu_a And 1) > 0 Then co = $10
					cpu_a = (cpu_a Shr 1) And $FF
					cpu_f = 0
					If cpu_a = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $38 ;SRL B
					cpu_m = 2
					If (cpu_b And 1) > 0 Then co = $10
					cpu_b = (cpu_b Shr 1) And $FF
					cpu_f = 0
					If cpu_b = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $39 ;SRL C
					cpu_m = 2
					If (cpu_c And 1) > 0 Then co = $10
					cpu_c = (cpu_c Shr 1) And $FF
					cpu_f = 0
					If cpu_c = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $3A ;SRL D
					cpu_m = 2
					If (cpu_d And 1) > 0 Then co = $10
					cpu_d = (cpu_d Shr 1) And $FF
					cpu_f = 0
					If cpu_d = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $3B ;SRL E
					cpu_m = 2
					If (cpu_e And 1) > 0 Then co = $10
					cpu_e = (cpu_e Shr 1) And $FF
					cpu_f = 0
					If cpu_e = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $3C ;SRL H
					cpu_m = 2
					If (cpu_h And 1) > 0 Then co = $10
					cpu_h = (cpu_h Shr 1) And $FF
					cpu_f = 0
					If cpu_h = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $3D ;SRL L
					cpu_m = 2
					If (cpu_l And 1) > 0 Then co = $10
					cpu_l = (cpu_l Shr 1) And $FF
					cpu_f = 0
					If cpu_l = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
				Case $3E ;SRL (HL)
					cpu_m = 4
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And 1) > 0 Then co = $10
					value = (value Shr 1) And $FF
					cpu_f = 0
					If value = 0 Then cpu_f = $80
					cpu_f = cpu_f Or co
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
					
				Case $47 ;BIT 0,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $40 ;BIT 0,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $41 ;BIT 0,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $42 ;BIT 0,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $43 ;BIT 0,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $44 ;BIT 0,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $45 ;BIT 0,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $01) = 0 Then cpu_f = cpu_f Or $80
				Case $46 ;BIT 0,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $01) = 0 Then cpu_f = cpu_f Or $80
					
				Case $4F ;BIT 1,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $48 ;BIT 1,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $49 ;BIT 1,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $4A ;BIT 1,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $4B ;BIT 1,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $4C ;BIT 1,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $4D ;BIT 1,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $02) = 0 Then cpu_f = cpu_f Or $80
				Case $4E ;BIT 1,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $02) = 0 Then cpu_f = cpu_f Or $80
					
				Case $57 ;BIT 2,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $50 ;BIT 2,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $51 ;BIT 2,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $52 ;BIT 2,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $53 ;BIT 2,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $54 ;BIT 2,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $55 ;BIT 2,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $04) = 0 Then cpu_f = cpu_f Or $80
				Case $56 ;BIT 2,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $04) = 0 Then cpu_f = cpu_f Or $80
					
				Case $5F ;BIT 3,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $58 ;BIT 3,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $59 ;BIT 3,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $5A ;BIT 3,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $5B ;BIT 3,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $5C ;BIT 3,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $5D ;BIT 3,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $08) = 0 Then cpu_f = cpu_f Or $80
				Case $5E ;BIT 3,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $08) = 0 Then cpu_f = cpu_f Or $80
					
				Case $67 ;BIT 4,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $10) = 0 Then cpu_f = cpu_f Or $80
				Case $60 ;BIT 4,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $10) = 0 Then cpu_f = cpu_f Or $80
				Case $61 ;BIT 4,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $10) = 0 Then cpu_f = cpu_f Or $80				
				Case $62 ;BIT 4,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $10) = 0 Then cpu_f = cpu_f Or $80		
				Case $63 ;BIT 4,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $10) = 0 Then cpu_f = cpu_f Or $80
				Case $64 ;BIT 4,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $10) = 0 Then cpu_f = cpu_f Or $80
				Case $65 ;BIT 4,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $10) = 0 Then cpu_f = cpu_f Or $80
				Case $66 ;BIT 4,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $10) = 0 Then cpu_f = cpu_f Or $80
					
				Case $6F ;BIT 5,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $68 ;BIT 5,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $69 ;BIT 5,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $6A ;BIT 5,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $6B ;BIT 5,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $6C ;BIT 5,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $6D ;BIT 5,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $20) = 0 Then cpu_f = cpu_f Or $80
				Case $6E ;BIT 5,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $20) = 0 Then cpu_f = cpu_f Or $80
					
				Case $77 ;BIT 6,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $70 ;BIT 6,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $71 ;BIT 6,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $72 ;BIT 6,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $73 ;BIT 6,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $74 ;BIT 6,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $75 ;BIT 6,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $40) = 0 Then cpu_f = cpu_f Or $80
				Case $76 ;BIT 6,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $40) = 0 Then cpu_f = cpu_f Or $80
					
				Case $7F ;BIT 7,A
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_a And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $78 ;BIT 7,B
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_b And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $79 ;BIT 7,C
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_c And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $7A ;BIT 7,D
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_d And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $7B ;BIT 7,E
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_e And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $7C ;BIT 7,H
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_h And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $7D ;BIT 7,L
					cpu_m = 2
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					If (cpu_l And $80) = 0 Then cpu_f = cpu_f Or $80
				Case $7E ;BIT 7,(HL)
					cpu_m = 3
					cpu_f = cpu_f And $1F
					cpu_f = cpu_f Or $20
					value = memory_read_byte((cpu_h Shl 8) + cpu_l)
					If (value And $80) = 0 Then cpu_f = cpu_f Or $80
					
					
				Case $C7 ;SET 0,A
					cpu_m = 2
					cpu_a = cpu_a Or $01
				Case $C0 ;SET 0,B
					cpu_m = 2
					cpu_b = cpu_b Or $01
				Case $C1 ;SET 0,C
					cpu_m = 2
					cpu_c = cpu_c Or $01
				Case $C2 ;SET 0,D
					cpu_m = 2
					cpu_d = cpu_d Or $01
				Case $C3 ;SET 0,E
					cpu_m = 2
					cpu_e = cpu_e Or $01
				Case $C4 ;SET 0,H
					cpu_m = 2
					cpu_h = cpu_h Or $01
				Case $C5 ;SET 0,L
					cpu_m = 2
					cpu_l = cpu_l Or $01
				Case $C6 ;SET 0,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or 1
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $CF ;SET 1,A
					cpu_m = 2
					cpu_a = cpu_a Or $02
				Case $C8 ;SET 1,B
					cpu_m = 2
					cpu_b = cpu_b Or $02
				Case $C9 ;SET 1,C
					cpu_m = 2
					cpu_c = cpu_c Or $02
				Case $CA ;SET 1,D
					cpu_m = 2
					cpu_d = cpu_d Or $02
				Case $CB ;SET 1,E
					cpu_m = 2
					cpu_e = cpu_e Or $02
				Case $CC ;SET 1,H
					cpu_m = 2
					cpu_h = cpu_h Or $02		
				Case $CD ;SET 1,L
					cpu_m = 2
					cpu_l = cpu_l Or $02
				Case $CE ;SET 1,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or 2
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $D7 ;SET 2,A
					cpu_m = 2
					cpu_a = cpu_a Or $04
				Case $D0 ;SET 2,B
					cpu_m = 2
					cpu_b = cpu_b Or $04
				Case $D1 ;SET 2,C
					cpu_m = 2
					cpu_c = cpu_c Or $04
				Case $D2 ;SET 2,D
					cpu_m = 2
					cpu_d = cpu_d Or $04
				Case $D3 ;SET 2,E
					cpu_m = 2
					cpu_e = cpu_e Or $04
				Case $D4 ;SET 2,H
					cpu_m = 2
					cpu_h = cpu_h Or $04
				Case $D5 ;SET 2,L
					cpu_m = 2
					cpu_l = cpu_l Or $04
				Case $D6 ;SET 2,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or 4
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $DF ;SET 3,A
					cpu_m = 2
					cpu_a = cpu_a Or $08
				Case $D8 ;SET 3,B
					cpu_m = 2
					cpu_b = cpu_b Or $08
				Case $D9 ;SET 3,C
					cpu_m = 2
					cpu_c = cpu_c Or $08
				Case $DA ;SET 3,D
					cpu_m = 2
					cpu_d = cpu_d Or $08
				Case $DB ;SET 3,E
					cpu_m = 2
					cpu_e = cpu_e Or $08
				Case $DC ;SET 3,H
					cpu_m = 2
					cpu_h = cpu_h Or $08
				Case $DD ;SET 3,L
					cpu_m = 2
					cpu_l = cpu_l Or $08
				Case $DE ;SET 3,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or 8
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $E7 ;SET 4,A
					cpu_m = 2
					cpu_a = cpu_a Or $10
				Case $E0 ;SET 4,B
					cpu_m = 2
					cpu_b = cpu_b Or $10
				Case $E1 ;SET 4,C
					cpu_m = 2
					cpu_c = cpu_c Or $10
				Case $E2 ;SET 4,D
					cpu_m = 2
					cpu_d = cpu_d Or $10
				Case $E3 ;SET 4,E
					cpu_m = 2
					cpu_e = cpu_e Or $10
				Case $E4 ;SET 4,H
					cpu_m = 2
					cpu_h = cpu_h Or $10
				Case $E5 ;SET 4,L
					cpu_m = 2
					cpu_l = cpu_l Or $10
				Case $E6 ;SET 4,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or $10
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $EF ;SET 5,A
					cpu_m = 2
					cpu_a = cpu_a Or $20
				Case $E8 ;SET 5,B
					cpu_m = 2
					cpu_b = cpu_b Or $20
				Case $E9 ;SET 5,C
					cpu_m = 2
					cpu_c = cpu_c Or $20
				Case $EA ;SET 5,D
					cpu_m = 2
					cpu_d = cpu_d Or $20
				Case $EB ;SET 5,E
					cpu_m = 2
					cpu_e = cpu_e Or $20
				Case $EC ;SET 5,H
					cpu_m = 2
					cpu_h = cpu_h Or $20
				Case $ED ;SET 5,L
					cpu_m = 2
					cpu_l = cpu_l Or $20
				Case $EE ;SET 5,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or $20
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $F7 ;SET 6,A
					cpu_m = 2
					cpu_a = cpu_a Or $40
				Case $F0 ;SET 6,B
					cpu_m = 2
					cpu_b = cpu_b Or $40
				Case $F1 ;SET 6,C
					cpu_m = 2
					cpu_c = cpu_c Or $40
				Case $F2 ;SET 6,D
					cpu_m = 2
					cpu_d = cpu_d Or $40
				Case $F3 ;SET 6,E
					cpu_m = 2
					cpu_e = cpu_e Or $40
				Case $F4 ;SET 6,H
					cpu_m = 2
					cpu_h = cpu_h Or $40
				Case $F5 ;SET 6,L
					cpu_m = 2
					cpu_l = cpu_l Or $40
				Case $F6 ;SET 6,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or $40
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $FF ;SET 7,A
					cpu_m = 2
					cpu_a = cpu_a Or $80
				Case $F8 ;SET 7,B
					cpu_m = 2
					cpu_b = cpu_b Or $80
				Case $F9 ;SET 7,C
					cpu_m = 2
					cpu_c = cpu_c Or $80
				Case $FA ;SET 7,D
					cpu_m = 2
					cpu_d = cpu_d Or $80
				Case $FB ;SET 7,E
					cpu_m = 2
					cpu_e = cpu_e Or $80
				Case $FC ;SET 7,H
					cpu_m = 2
					cpu_h = cpu_h Or $80
				Case $FD ;SET 7,L
					cpu_m = 2
					cpu_l = cpu_l Or $80
				Case $FE ;SET 7,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) Or $80
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)


				Case $87 ;RES 0,A
					cpu_m = 2
					cpu_a = cpu_a And $FE
				Case $80 ;RES 0,B
					cpu_m = 2
					cpu_b = cpu_b And $FE
				Case $81 ;RES 0,C
					cpu_m = 2
					cpu_c = cpu_c And $FE
				Case $82 ;RES 0,D
					cpu_m = 2
					cpu_d = cpu_d And $FE
				Case $83 ;RES 0,E
					cpu_m = 2
					cpu_e = cpu_e And $FE
				Case $84 ;RES 0,H
					cpu_m = 2
					cpu_h = cpu_h And $FE
				Case $85 ;RES 0,L
					cpu_m = 2
					cpu_l = cpu_l And $FE
				Case $86 ;RES 0,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $FE
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $8F ;RES 1,A
					cpu_m = 2
					cpu_a = cpu_a And $FD
				Case $88 ;RES 1,B
					cpu_m = 2
					cpu_b = cpu_b And $FD
				Case $89 ;RES 1,C
					cpu_m = 2
					cpu_c = cpu_c And $FD
				Case $8A ;RES 1,D
					cpu_m = 2
					cpu_d = cpu_d And $FD
				Case $8B ;RES 1,E
					cpu_m = 2
					cpu_e = cpu_e And $FD
				Case $8C ;RES 1,H
					cpu_m = 2
					cpu_h = cpu_h And $FD
				Case $8D ;RES 1,L
					cpu_m = 2
					cpu_l = cpu_l And $FD
				Case $8E ;RES 1,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $FD
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $97 ;RES 2,A
					cpu_m = 2
					cpu_a = cpu_a And $FB
				Case $90 ;RES 2,B
					cpu_m = 2
					cpu_b = cpu_b And $FB
				Case $91 ;RES 2,C
					cpu_m = 2
					cpu_c = cpu_c And $FB
				Case $92 ;RES 2,D
					cpu_m = 2
					cpu_d = cpu_d And $FB
				Case $93 ;RES 2,E
					cpu_m = 2
					cpu_e = cpu_e And $FB
				Case $94 ;RES 2,H
					cpu_m = 2
					cpu_h = cpu_h And $FB
				Case $95 ;RES 2,L
					cpu_m = 2
					cpu_l = cpu_l And $FB
				Case $96 ;RES 2,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $FB
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $9F ;RES 3,A
					cpu_m = 2
					cpu_a = cpu_a And $F7
				Case $98 ;RES 3,B
					cpu_m = 2
					cpu_b = cpu_b And $F7
				Case $99 ;RES 3,C
					cpu_m = 2
					cpu_c = cpu_c And $F7
				Case $9A ;RES 3,D
					cpu_m = 2
					cpu_d = cpu_d And $F7
				Case $9B ;RES 3,E
					cpu_m = 2
					cpu_e = cpu_e And $F7
				Case $9C ;RES 3,H
					cpu_m = 2
					cpu_h = cpu_h And $F7
				Case $9D ;RES 3,L
					cpu_m = 2
					cpu_l = cpu_l And $F7
				Case $9E ;RES 3,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $F7
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $A7 ;RES 4,A
					cpu_m = 2
					cpu_a = cpu_a And $EF
				Case $A0 ;RES 4,B
					cpu_m = 2
					cpu_b = cpu_b And $EF
				Case $A1 ;RES 4,C
					cpu_m = 2
					cpu_c = cpu_c And $EF
				Case $A2 ;RES 4,D
					cpu_m = 2
					cpu_d = cpu_d And $EF
				Case $A3 ;RES 4,E
					cpu_m = 2
					cpu_e = cpu_e And $EF
				Case $A4 ;RES 4,H
					cpu_m = 2
					cpu_h = cpu_h And $EF
				Case $A5 ;RES 4,L
					cpu_m = 2
					cpu_l = cpu_l And $EF
				Case $A6 ;RES 4,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $EF
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $AF ;RES 5,A
					cpu_m = 2
					cpu_a = cpu_a And $DF
				Case $A8 ;RES 5,B
					cpu_m = 2
					cpu_b = cpu_b And $DF
				Case $A9 ;RES 5,C
					cpu_m = 2
					cpu_c = cpu_c And $DF
				Case $AA ;RES 5,D
					cpu_m = 2
					cpu_d = cpu_d And $DF
				Case $AB ;RES 5,E
					cpu_m = 2
					cpu_e = cpu_e And $DF
				Case $AC ;RES 5,H
					cpu_m = 2
					cpu_h = cpu_h And $DF
				Case $AD ;RES 5,L
					cpu_m = 2
					cpu_l = cpu_l And $DF
				Case $AE ;RES 5,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $DF
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $B7 ;RES 6,A
					cpu_m = 2
					cpu_a = cpu_a And $BF
				Case $B0 ;RES 6,B
					cpu_m = 2
					cpu_b = cpu_b And $BF
				Case $B1 ;RES 6,C
					cpu_m = 2
					cpu_c = cpu_c And $BF
				Case $B2 ;RES 6,D
					cpu_m = 2
					cpu_d = cpu_d And $BF
				Case $B3 ;RES 6,E
					cpu_m = 2
					cpu_e = cpu_e And $BF
				Case $B4 ;RES 6,H
					cpu_m = 2
					cpu_h = cpu_h And $BF
				Case $B5 ;RES 6,L
					cpu_m = 2
					cpu_l = cpu_l And $BF
				Case $B6 ;RES 6,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $BF
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)

				Case $BF ;RES 7,A
					cpu_m = 2
					cpu_a = cpu_a And $7F
				Case $B8 ;RES 7,B
					cpu_m = 2
					cpu_b = cpu_b And $7F
				Case $B9 ;RES 7,C
					cpu_m = 2
					cpu_c = cpu_c And $7F
				Case $BA ;RES 7,D
					cpu_m = 2
					cpu_d = cpu_d And $7F
				Case $BB ;RES 7,E
					cpu_m = 2
					cpu_e = cpu_e And $7F
				Case $BC ;RES 7,H
					cpu_m = 2
					cpu_h = cpu_h And $7F
				Case $BD ;RES 7,L
					cpu_m = 2
					cpu_l = cpu_l And $7F
				Case $BE ;RES 7,(HL)
					cpu_m = 3
					value = memory_read_byte((cpu_h Shl 8) + cpu_l) And $7F
					memory_write_byte((cpu_h Shl 8) + cpu_l, value)
					
				Default
					Notify "Unknown Opcode: $CB " + Hex$(opcode), 1
					End
					
			End Select
			
			
		;8Bit DAA
		Case $27 ;DAA
			cpu_m = 1
			a = cpu_a
			
			If (cpu_f And $40) = 0 Then
				If (cpu_f And $20) > 0 Or (a And $0F) > 9 Then a = a + $06
				If (cpu_f And $10) > 0 Or a > $9F Then a = a + $60
			Else
				If (cpu_f And $20) > 0 Then a = (a - $06) And $FF
				If (cpu_f And $10) > 0 Then a = a - $60
			End If
			
			cpu_f = cpu_f And $5F
			If (a And $100) = $100 Then cpu_f = cpu_f Or $10
			
			a = a And $FF
			If a = 0 Then
				cpu_f = cpu_f Or $80
			Else
				cpu_f = cpu_f And $7F
			End If
			
			cpu_a = a
			
		;8Bit CPL
		Case $2F ;CPL
			cpu_m = 1
			cpu_a = cpu_a Xor $FF
			cpu_f = cpu_f Or $60
			
			
		;1Bit CCF
		Case $3F ;CCF
			cpu_m = 1
			If (cpu_f And $10) > 0 Then
				cpu_f = cpu_f And $EF
			Else
				cpu_f = cpu_f Or $10
			EndIf
			cpu_f = cpu_f And $9F
			
						
		;1Bit SCF
		Case $37 ;SCF
			cpu_m = 1
			cpu_f = cpu_f Or $10
			cpu_f = cpu_f And $9F
		
		
		;NOP
		Case $00 ;NOP
			cpu_m = 1
		
		
		;8Bit RLCA
		Case $07 ;RLCA
			cpu_m = 1
			cpu_f = 0
			If (cpu_a And $80) > 0 Then
				cpu_f = cpu_f Or $10
				add = 1
			End If
			cpu_a = ((cpu_a Shl 1) + add) And $FF
			
		;8Bit RLA
		Case $17 ;RLA
			cpu_m = 1
			If (cpu_f And $10) > 0 Then add = 1
			cpu_f = 0
			If (cpu_a And $80) > 0 Then cpu_f = $10
			cpu_a = ((cpu_a Shl 1) + add) And $FF
		
		;8Bit RRCA
		Case $0F ;RRCA
			cpu_m = 1
			cpu_f = 0
			If (cpu_a And 1) > 0 Then
				cpu_f = cpu_f Or $10
				add = $80
			End If
			cpu_a = ((cpu_a Shr 1) + add) And $FF
		
		;8Bit RRA
		Case $1F ;RRA
			cpu_m = 1
			If (cpu_f And $10) > 0 Then add = $80
			cpu_f = 0
			If (cpu_a And 1) > 0 Then cpu_f = $10
			cpu_a = ((cpu_a Shr 1) + add) And $FF
			
			
		;16Bit JP
		Case $C3 ;JP XXYY
			cpu_m = 3
			cpu_pc = memory_read_word(cpu_pc)
			
		Case $C2 ;JP NZ,XXYY
			If (Not cpu_f And $80) > 0 Then 
				cpu_m = 4
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $CA ;JP Z,XXYY
			If (cpu_f And $80) > 0 Then 
				cpu_m = 4
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $D2 ;JP NC,XXYY
			If (Not cpu_f And $10) > 0 Then 
				cpu_m = 4
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $DA ;JP C,XXYY
			If (cpu_f And $10) > 0 Then 
				cpu_m = 4
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $E9 ;JP HL
			cpu_m = 1
			cpu_pc = (cpu_h Shl 8) + cpu_l
			
			
		;8Bit JR
		Case $18 ;JR XX
			cpu_m = 3
			value = memory_read_byte(cpu_pc)
			If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
			cpu_pc = cpu_pc + value + 1

		Case $20 ;JR NZ,XX
			If (Not cpu_f And $80) > 0 Then 
				cpu_m = 3
				value = memory_read_byte(cpu_pc)
				If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
				cpu_pc = cpu_pc + value + 1
			Else
				cpu_m = 2
				cpu_pc = cpu_pc+1
			End If
			
		Case $28 ;JR Z,XX
			If (cpu_f And $80) > 0 Then 
				cpu_m = 3
				value = memory_read_byte(cpu_pc)
				If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
				cpu_pc = cpu_pc + value + 1
			Else
				cpu_m = 2
				cpu_pc = cpu_pc+1
			End If
			
		Case $30 ;JR NC,XX
			If (Not cpu_f And $10) > 0 Then 
				cpu_m = 3
				value = memory_read_byte(cpu_pc)
				If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
				cpu_pc = cpu_pc + value + 1
			Else
				cpu_m = 2
				cpu_pc = cpu_pc+1
			End If
			
		Case $38 ;JR C,XX
			If (cpu_f And $10) > 0 Then 
				cpu_m = 3
				value = memory_read_byte(cpu_pc)
				If value > $7F Then value = (-((value Xor $FF) And $FF)-1); Complement
				cpu_pc = cpu_pc + value + 1
			Else
				cpu_m = 2
				cpu_pc = cpu_pc+1
			End If
			
			
		;16Bit CALL
		Case $CD ;CALL XXYY
			cpu_m = 5
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc+2)
			cpu_pc = memory_read_word(cpu_pc)
			
		Case $C4 ;CALL NZ,XXYY
			If (Not cpu_f And $80) > 0 Then 
				cpu_m = 5
				cpu_sp = cpu_sp -2
				memory_write_word(cpu_sp, cpu_pc+2)
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $CC ;CALL Z,XXYY
			If (cpu_f And $80) > 0 Then 
				cpu_m = 5
				cpu_sp = cpu_sp -2
				memory_write_word(cpu_sp, cpu_pc+2)
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $D4 ;CALL NC,XXYY
			If (Not cpu_f And $10) > 0 Then 
				cpu_m = 5
				cpu_sp = cpu_sp -2
				memory_write_word(cpu_sp, cpu_pc+2)
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
		Case $DC ;CALL C,XXYY
			If (cpu_f And $10) > 0 Then 
				cpu_m = 5
				cpu_sp = cpu_sp -2
				memory_write_word(cpu_sp, cpu_pc+2)
				cpu_pc = memory_read_word(cpu_pc)
			Else
				cpu_m = 3
				cpu_pc = cpu_pc+2
			End If
			
			
		;8Bit RST
		Case $C7 ;RST $00
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $00
			
		Case $CF ;RST $08
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $08
			
		Case $D7 ;RST $10
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $10
			
		Case $DF ;RST $18
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $18

		Case $E7 ;RST $20
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $20
			
		Case $EF ;RST $28
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $28
			
		Case $F7 ;RST $30
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $30
			
		Case $FF ;RST $38
			cpu_m = 3
			cpu_sp = cpu_sp -2
			memory_write_word(cpu_sp, cpu_pc)
			cpu_pc = $38
			
			
		;16Bit RET
		Case $C9 ;RET
			cpu_m = 3
			cpu_pc = memory_read_word(cpu_sp)
			cpu_sp = cpu_sp + 2
			
		Case $C0 ;RET NZ
			If (Not cpu_f And $80) > 0 Then 
				cpu_m = 3
				cpu_pc = memory_read_word(cpu_sp)
				cpu_sp = cpu_sp + 2
			Else
				cpu_m = 1
			End If
			
		Case $C8 ;RET Z
			If (cpu_f And $80) > 0 Then 
				cpu_m = 3
				cpu_pc = memory_read_word(cpu_sp)
				cpu_sp = cpu_sp + 2
			Else
				cpu_m = 1
			End If
			
		Case $D0 ;RET NC
			If (Not cpu_f And $10) > 0 Then 
				cpu_m = 3
				cpu_pc = memory_read_word(cpu_sp)
				cpu_sp = cpu_sp + 2
			Else
				cpu_m = 1
			End If

		Case $D8 ;RET C
			If (cpu_f And $10) > 0 Then 
				cpu_m = 3
				cpu_pc = memory_read_word(cpu_sp)
				cpu_sp = cpu_sp + 2
			Else
				cpu_m = 1
			End If
			
			
		Case $F3 ;DI
			cpu_m = 1
			cpu_ime = 0
			
		Case $FB ;EI
			cpu_m = 1
			cpu_ime = 1
			
		Case $76 ;HALT
			cpu_m = 1
			If cpu_ime = 0 Then cpu_pc = cpu_pc + 1
			cpu_halt = 1
			
		Case $10 ;STOP
			opcode2 = memory_read_byte(cpu_pc)
			cpu_pc = cpu_pc + 1
			If opcode2 = $00 Then
				cpu_stop = 1
			End If
			
		Case $D9 ;RETI
			cpu_m = 3
			cpu_pc = memory_read_word(cpu_sp)
			cpu_sp = cpu_sp + 2
			cpu_ime = 1
			
		Default
			Notify "Unknown Opcode: " + Hex$(opcode), 1
			End
	End Select
End Function

Function cpu_rst(adr)
	cpu_m = 3
	cpu_sp = cpu_sp -2
	memory_write_word(cpu_sp, cpu_pc)
	cpu_pc = adr
End Function