;gpu.bb
;BlitzBoy
;
;Created by Jan Ebbe on 05.01.2012


Dim gpu_buffer(159,143)

Dim gpu_pal(3)
gpu_pal(3) = $FF111111
gpu_pal(2) = $FF666666
gpu_pal(1) = $FFAAAAAA
gpu_pal(0) = $FFEEEEEE

gpu_pal(3) = $FF204631
gpu_pal(2) = $FF527F39
gpu_pal(1) = $FFAEC440
gpu_pal(0) = $FFD7E894

Global gpu_mode
Global gpu_modeclock
Global gpu_line
Global gpu_bgmap
Global gpu_scy
Global gpu_scx
Global gpu_bgtile
Global gpu_switchbg
Global gpu_switchlcd
Global gpu_switchobj
Global gpu_compare
Global gpu_objsize
Global gpu_switchwin
Global gpu_winmap
Global gpu_wx
Global gpu_wy
Global gpu_ie_compare
Global gpu_ie_oam
Global gpu_ie_vblank
Global gpu_ie_hblank

Dim gpu_tileset(384,7,7)
Dim gpu_bgpal(3)

Dim gpu_sprites(39,6)
;0 = Y
;1 = X
;2 = Tile
;3 = Palette
;4 = X Flip
;5 = Y Flip
;6 = Prio

Dim gpu_objpal0(3)
Dim gpu_objpal1(3)


Function gpu_reset()
	gpu_mode = 0
	gpu_modeclock = 0
	gpu_line = 0
	gpu_bgmap = 0
	gpu_scy = 0
	gpu_scx = 0
	gpu_bgtile = 0
	gpu_switchbg = 0
	gpu_switchlcd = 0
	gpu_compare = 0
	gpu_objsize = 0
	gpu_switchwin = 0
	gpu_winmap = 0
	gpu_wy = 0
	gpu_wx = 0
	gpu_ie_compare = 0
	gpu_ie_oam = 0
	gpu_ie_vblank = 0
	gpu_ie_hblank = 0

	Dim gpu_tileset(384,7,7)
	Dim gpu_bgpal(3)
	Dim gpu_sprites(39,7)
	Dim gpu_objpal0(3)
	Dim gpu_objpal1(3)
End Function

Function gpu_step()
	gpu_modeclock = gpu_modeclock + cpu_m
	
	Select gpu_mode
	
		Case 2 ;OAM read mode, scanline active
			If gpu_modeclock >= 20 Then
				gpu_modeclock = 0
				gpu_mode = 3
			End If
			
		Case 3 ;VRAM read mode, scanline active
			If gpu_modeclock >= 43 Then
				gpu_modeclock = 0
				gpu_mode = 0
				
				gpu_render_scan()
			End If
			
		Case 0 ;Hblank
			If gpu_modeclock >= 51 Then
				If gpu_line = 143 Then
					gpu_mode = 1
					
					gpu_render_frame()
					
					memory_if = memory_if Or $01 ;V-Blank interrupt
				Else
					gpu_mode = 2
				End If
				
				gpu_modeclock = 0
				gpu_line = gpu_line + 1
				
				If gpu_ie_hblank = 1 Then memory_if = memory_if Or $02 ;H-Blank interrupt
			End If
			
		Case 1
			If gpu_modeclock >= 114 Then
				gpu_modeclock = 0
				gpu_line = gpu_line + 1
				
				If gpu_line > 153 Then
					gpu_mode = 2
					gpu_line = 0
				End If
			End If
	End Select
	
	If gpu_ie_compare = 1 And gpu_line = gpu_compare Then memory_if = memory_if Or $02 ;Compare interrupt
End Function

Function gpu_render_frame()
	SetBuffer CanvasBuffer(gui_canvas)
	LockBuffer CanvasBuffer(gui_canvas)
	
	For x=0 To 159
		For y=0 To 143
			sx = x Shl 1
			sy = y Shl 1
			
			Select gpu_buffer(x,y) And $C0
			
				Case $00
					WritePixelFast sx,sy,gpu_bgpal(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy,gpu_bgpal(gpu_buffer(x,y) And $03)
					WritePixelFast sx,sy+1,gpu_bgpal(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy+1,gpu_bgpal(gpu_buffer(x,y) And $03)
					
				Case $40
					WritePixelFast sx,sy,gpu_objpal0(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy,gpu_objpal0(gpu_buffer(x,y) And $03)
					WritePixelFast sx,sy+1,gpu_objpal0(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy+1,gpu_objpal0(gpu_buffer(x,y) And $03)
					
				Case $80
					WritePixelFast sx,sy,gpu_objpal1(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy,gpu_objpal1(gpu_buffer(x,y) And $03)
					WritePixelFast sx,sy+1,gpu_objpal1(gpu_buffer(x,y) And $03)
					WritePixelFast sx+1,sy+1,gpu_objpal1(gpu_buffer(x,y) And $03)
			End Select
		Next
	Next
	
	UnlockBuffer CanvasBuffer(gui_canvas)
	
	FlipCanvas(gui_canvas,0)
End Function

Function gpu_render_scan()
	;Render Background
	If gpu_switchbg = 1 Then
		mapoffs = $1800
		If gpu_bgmap = 1 Then mapoffs = $1C00
		
		mapoffs = mapoffs + ((gpu_line + gpu_scy) And $F8) Shl 2
		lineoffs = gpu_scx Shr 3
		
		y = (gpu_line + gpu_scy) And 7
		x = gpu_scx And 7
			
		tile = PeekByte (memory_vram, mapoffs+lineoffs)
		If gpu_bgtile = 1 And tile < 128 Then tile = tile + 256
	
		For i=0 To 159
			gpu_buffer(i,gpu_line) = gpu_tileset(tile,y,x)
			
			x = x + 1
			If x = 8 Then
				x = 0
				lineoffs = (lineoffs + 1) And 31
				tile = PeekByte (memory_vram, mapoffs+lineoffs)
				If gpu_bgtile = 1 And tile < 128 Then tile = tile + 256
			End If
		Next
	Else
		For i=0 To 159
			gpu_buffer(i,gpu_line) = 0
		Next
	End If
	
	;Render Window
	If gpu_switchwin = 1 And gpu_wx < 166 And gpu_line >= gpu_wy Then
		winoffs = $1800
		If gpu_winmap = 1 Then winoffs = $1C00
		
		winline = gpu_line-gpu_wy
		winoffs = winoffs + (winline And $F8) Shl 2
		
		tile = PeekByte (memory_vram, winoffs)
		If gpu_bgtile = 1 And tile < 128 Then tile = tile + 256
		x = 0
		y = winline And 7
		
		For i=gpu_wx-7 To 159
			If i >= 0 Then gpu_buffer(i,gpu_line) = gpu_tileset(tile,y,x)
			
			x = x + 1
			If x = 8 Then
				x = 0
				winoffs = winoffs + 1
				tile = PeekByte (memory_vram, winoffs)
				If gpu_bgtile = 1 And tile < 128 Then tile = tile + 256
			End If
		Next
	End If
	
	;Render Sprites
	If gpu_switchobj = 1 Then
		For i=0 To 39
			tile = -1
			
			If gpu_sprites(i,0) <= gpu_line And (gpu_sprites(i,0) + 8) > gpu_line Then
					If gpu_objsize = 0 Then ;8x8
						tile = gpu_sprites(i,2)
					Else ;8x16 Upper
						If gpu_sprites(i,5) = 1 Then
							tile = gpu_sprites(i,2) Or $01
						Else
							tile = gpu_sprites(i,2) And $FE
						End If
					End If
					tiley = gpu_sprites(i,0)
			Else If gpu_objsize = 1 And (gpu_sprites(i,0) + 8) <= gpu_line And (gpu_sprites(i,0) + 16) > gpu_line Then ;8x16 Lower
					If gpu_sprites(i,5) = 1 Then
						tile = gpu_sprites(i,2) And $FE
					Else
						tile = gpu_sprites(i,2) Or $01
					End If
					tiley = gpu_sprites(i,0) + 8
			End If
			
			If tile <> -1 Then
				If gpu_sprites(i,5) = 1 Then
					tilerow = 7 - (gpu_line - tiley)
				Else
					tilerow = gpu_line - tiley
				End If
				
				
				For x=0 To 7
					If gpu_sprites(i,4) = 1 Then
						rx = 7-x
					Else
						rx = x
					End If
				
					If (gpu_sprites(i,1)+rx) >= 0 And (gpu_sprites(i,1)+x) < 160 And gpu_tileset(tile,tilerow,rx) > 0 And (gpu_sprites(i,6) = 0 Or gpu_buffer(gpu_sprites(i,1)+rx,gpu_line) = 0) Then		
						If gpu_sprites(i,3) = 1 Then ;ObjPal 1
							gpu_buffer(x+gpu_sprites(i,1),gpu_line) = gpu_tileset(tile,tilerow,rx) Or $80
						Else ;ObjPal 0
							gpu_buffer(x+gpu_sprites(i,1),gpu_line) = gpu_tileset(tile,tilerow,rx) Or $40
						End If
					End If
				Next
			End If
		Next
	End If
End Function

Function gpu_update_tile(adr)
	adr = adr And $1FFE
	
	tile = (adr Shr 4) And $1FF
	y = (adr Shr 1) And $07
	
	For x=0 To 7
		sx = $01 Shl (7-x)
		
		v = 0
		If (PeekByte (memory_vram, adr) And sx) > 0 Then v = v + 1
		If (PeekByte (memory_vram, adr+1) And sx) > 0 Then v = v + 2
		
		gpu_tileset(tile,y,x) = v
	Next
End Function

Function gpu_update_sprite(adr, val)
	obj = adr Shr 2
	
	If obj < 40 Then
		Select (adr And $03)
	
			Case 0
				gpu_sprites(obj,0) = val-16
				
			Case 1
				gpu_sprites(obj,1) = val-8
				
			Case 2
				gpu_sprites(obj,2) = val
				
			Case 3
				gpu_sprites(obj,3) = 0
				gpu_sprites(obj,4) = 0
				gpu_sprites(obj,5) = 0
				gpu_sprites(obj,6) = 0
				
				If (val And $10) > 0 Then gpu_sprites(obj,3) = 1
				If (val And $20) > 0 Then gpu_sprites(obj,4) = 1
				If (val And $40) > 0 Then gpu_sprites(obj,5) = 1
				If (val And $80) > 0 Then gpu_sprites(obj,6) = 1
				
		End Select
	End If
End Function