;memory.bb
;BlitzBoy
;
;Created by Jan Ebbe on 05.01.2012


;Rom $0000 - $3FFF (bank 0), $4000 - $7FFF (bank n)
Global memory_rom = CreateBank(0)
Global memory_rom_size
Global memory_rom_type
Global memory_rom_banks
Global memory_rom_name$
Global memory_rom_path$

;Video Ram $8000 - $9FFF
Global memory_vram = CreateBank(8192)

;External Ram $A000 - $BFFF
Global memory_eram = CreateBank(8192)

;Working Ram $C000 - $DFFF (copied on $E000 - $FDFF)
Global memory_wram = CreateBank(8192)

;Graphics Sprite Information $FE00 - $FE9F
Global memory_oam = CreateBank(160)

;Graphics And Lcd Information $FF00 - $FF7F
Global memory_gpu = CreateBank(128)

;Zero-Page Ram $FF80 - $FFFF
Global memory_zram = CreateBank(128)
Global memory_ie
Global memory_if

;Memory Bank Controller
Global memory_rom_offset
Global memory_rom_bank
Global memory_mode


Function memory_reset()
	memory_rom = CreateBank(0)
	memory_rom_size = 0
	memory_rom_type = 0
	memory_rom_banks = 0
	memory_rom_name$ = ""
	memory_rom_path$ = ""
	
	memory_vram = CreateBank(8192)
	memory_eram = CreateBank(8192)
	memory_wram = CreateBank(8192)
	memory_oam = CreateBank(160)
	memory_gpu = CreateBank(128)
	
	memory_zram = CreateBank(128)
	memory_ie = 0
	memory_if = 0
	
	memory_rom_offset = 0
	memory_rom_bank = 0
	memory_mode = 0
End Function

Function memory_read_byte(adr)
	If adr <= $3FFF Then ;Rom Bank 0
		Return PeekByte (memory_rom, adr)
		
	Else If adr <= $7FFF ;Rom Bank n
		Return PeekByte (memory_rom, adr + memory_rom_offset) 
		
	Else If adr <= $9FFF ;Video Ram
		Return PeekByte (memory_vram, adr-$8000)
		
	Else If adr <= $BFFF ;External Ram
		Return PeekByte (memory_eram, adr-$A000)
		
	Else If adr <= $DFFF ;Working Ram
		Return PeekByte (memory_wram, adr-$C000)
		
	Else If adr <= $FDFF ;Working Ram Copy
		Return PeekByte (memory_wram, adr-$E000)
		
	Else If adr <= $FE9F ;Graphics Sprite Information
		Return PeekByte (memory_oam, adr-$FE00)
	Else If adr <= $FEFF ;Not Usable
		;Not Usable
	Else If adr <= $FF7F ;Memory Mapped I/O
		If adr = $FF00 Then ;P1 - Keypad	
			If keypad_column = $10 Then Return keypad_rows (0)
			If keypad_column = $20 Then Return keypad_rows (1)
			Return 0
		Else If adr = $FF04 Then ;DIV
			Return timer_div
		Else If adr = $FF05 Then ;TIMA
			Return timer_tima
		Else If adr = $FF06 Then ;TMA
			Return timer_tma
		Else If adr = $FF07 Then ;TAC
			Return timer_tac
		Else If adr = $FF0F Then ;IF - Interrupt Flag
			Return memory_if
		Else If adr = $FF26 Then ;NR52
			If sound_enabled = 0 Then
				Return $00
			Else
				value = value Or $40
				If sound_m1_playing = 1 Then value = value Or $01
				If sound_m2_playing = 1 Then value = value Or $02
				If sound_m3_playing = 1 Then value = value Or $04
				If sound_m4_playing = 1 Then value = value Or $08
				Return value
			End If
		Else If adr = $FF41 Then ;STAT - LCDC Status
			value = gpu_mode
			If gpu_line = gpu_compare Then value = value Or $04
			If gpu_ie_hblank = 1 Then value = value Or $08
			If gpu_ie_vblank = 1 Then value = value Or $10
			If gpu_ie_oam = 1 Then value = value Or $20
			If gpu_ie_compare = 1 Then value = value Or $40
			Return value
		Else If adr = $FF44 Then ;LY - LCDC Y-Coordinate
			Return gpu_line
		Else If adr = $FF45 Then ;LYC - LY Compare
			Return gpu_compare
		End If
		
		Return PeekByte (memory_gpu, adr-$FF00)
		
	Else If adr <= $FFFF ;Zero-Page Ram
		If adr = $FFFF Then ;IE - Interrupt Enable
			Return memory_ie
		End If
		
		Return PeekByte (memory_zram, adr-$FF80)
	
	Else
		Notify "Unknown Memory Bank: " + Hex$(adr), 1
		End
	
	End If
End Function

Function memory_read_word(adr)
	Return memory_read_byte(adr) + (memory_read_byte(adr+1) Shl 8)
End Function

Function memory_write_byte(adr,val)
	If adr <= $7FFF ;Memory Bank Controller
		memory_bank_controller(adr,val)
	Else If adr <= $9FFF ;Video Ram
		PokeByte (memory_vram, adr-$8000, val)
		If adr <= $97FF Then gpu_update_tile(adr) ;Update Tile
		
	Else If adr <= $BFFF ;External Ram
		PokeByte (memory_eram, adr-$A000, val)
		
	Else If adr <= $DFFF ;Working Ram
		PokeByte (memory_wram, adr-$C000, val)
		
	Else If adr <= $FDFF ;Working Ram Copy
		PokeByte (memory_wram, adr-$E000, val)
		
	Else If adr <= $FE9F ;Graphics Sprite Information
		PokeByte (memory_oam, adr-$FE00, val)
		gpu_update_sprite(adr-$FE00, val) ;Update Sprite
		
	Else If adr <= $FEFF ;Not Usable
		;Not Usable
		
	Else If adr <= $FF7F ;Memory Mapped I/O
		PokeByte (memory_gpu, adr-$FF00, val)
		
		If adr = $FF00 Then ;P1 - Keypad
			keypad_column = val And $30
		Else If adr = $FF04 Then ;DIV
			timer_div = 0
		Else If adr = $FF05 Then ;TIMA
			timer_tima = val
		Else If adr = $FF06 Then ;TMA
			timer_tma = val
		Else If adr = $FF07 Then ;TAC
			timer_tac = val And $07
		Else If adr = $FF0F Then ;IF - Interrupt Flag
			memory_if = val
		Else If adr = $FF10 Then ;NR10
			sound_m1_sweep_shifts = val And $07
			sound_m1_sweep_direction = (val Shr 3) And $01
			sound_m1_sweep_time = ((val Shr 4) And $07) Shl 13
		Else If adr = $FF11 Then ;NR11
			sound_m1_length = (64 - (val And $3F)) Shl 12
			sound_m1_wave_duty = (val Shr 6) And $03
		Else If adr = $FF12 Then ;NR12
			sound_m1_envelope_initial_volume = (val Shr 4) And $0F
			sound_m1_envelope_direction = (val Shr 3) And $01
			sound_m1_envelope_time = (val And $07) Shl 14
		Else If adr = $FF13 Then ;NR13
			sound_m1_frequency_data = (sound_m1_frequency_data And $300) + val
		Else If adr = $FF14 Then ;NR14
			sound_m1_frequency_data = (sound_m1_frequency_data And $FF) + (val And $07) Shl 8
			sound_m1_consecutive = 0
			If (val And $40) > 0 Then sound_m1_consecutive = 1
			If (val And $80) > 0 Then sound_m1_start()
		Else If adr = $FF16 Then ;NR21
			sound_m2_length = (64 - (val And $3F)) Shl 12
			sound_m2_wave_duty = (val Shr 6) And $03
		Else If adr = $FF17 Then ;NR22
			sound_m2_envelope_initial_volume = (val Shr 4) And $0F
			sound_m2_envelope_direction = (val Shr 3) And $01
			sound_m2_envelope_time = (val And $07) Shl 14
		Else If adr = $FF18 Then ;NR23
			sound_m2_frequency_data = (sound_m2_frequency_data And $300) + val
		Else If adr = $FF19 Then ;NR24
			sound_m2_frequency_data = (sound_m2_frequency_data And $FF) + (val And $07) Shl 8
			sound_m2_consecutive = 0
			If (val And $40) > 0 Then sound_m2_consecutive = 1
			If (val And $80) > 0 Then sound_m2_start()
		Else If adr = $FF1A Then ;NR30
			sound_m3_output = 0
			If (val And $80) > 0 Then sound_m3_output = 1
		Else If adr = $FF1B Then ;NR31
			sound_m3_length = (256 - val) Shl 12
		Else If adr = $FF1C Then ;NR32
			sound_m3_volume = (val Shr 5) And $03
			If sound_m3_volume > 0 Then sound_m3_volume = $0F Shr sound_m3_volume
		Else If adr = $FF1D Then ;N33
			sound_m3_frequency_data = (sound_m3_frequency_data And $300) + val
		Else If adr = $FF1E Then ;NR34
			sound_m3_frequency_data = (sound_m3_frequency_data And $FF) + (val And $07) Shl 8
			sound_m3_consecutive = 0
			If (val And $40) > 0 Then sound_m3_consecutive = 1
			If (val And $80) > 0 Then sound_m3_start()
		Else If adr = $FF20 Then ;NR41
			sound_m4_length = (64 - (val And $3F)) Shl 12
		Else If adr = $FF21 Then ;NR42
			sound_m4_envelope_initial_volume = (val Shr 4) And $0F
			sound_m4_envelope_direction = (val Shr 3) And $01
			sound_m4_envelope_time = (val And $07) Shl 14
		Else If adr = $FF22 Then ;NR43
			sound_m4_shift_clock_frequency = (val Shr 4) And $0F
			sound_m4_polynomial_counter_step = (val Shr 3) And $01
			sound_m4_frequency_ratio = val And $07
		Else If adr = $FF23 Then ;NR44
			sound_m4_consecutive = 0
			If (val And $40) > 0 Then sound_m4_consecutive = 1
			If (val And $80) > 0 Then sound_m4_start()
		Else If adr = $FF25 Then ;NR51
			sound_m1_so1_output = 0
			sound_m1_so2_output = 0
			sound_m2_so1_output = 0
			sound_m2_so2_output = 0
			sound_m3_so1_output = 0
			sound_m3_so2_output = 0
			sound_m4_so1_output = 0
			sound_m4_so2_output = 0
			If (val And $01) > 0 Then sound_m1_so1_output = 1
			If (val And $02) > 0 Then sound_m2_so1_output = 1
			If (val And $04) > 0 Then sound_m3_so1_output = 1
			If (val And $08) > 0 Then sound_m4_so1_output = 1
			If (val And $10) > 0 Then sound_m1_so2_output = 1
			If (val And $20) > 0 Then sound_m2_so2_output = 1
			If (val And $40) > 0 Then sound_m3_so2_output = 1
			If (val And $80) > 0 Then sound_m4_so2_output = 1
		Else If adr = $FF26 Then ;NR52
			sound_enabled = 0
			If (val And $80) > 0 Then sound_enabled = 1
		Else If adr >= $FF30 And adr <= $FF3F ;WAVEFORM
			sound_update_waveform(adr, val)
		Else If adr = $FF40 Then ;LCDC - LCD Control
			gpu_switchbg = 0
			gpu_switchobj = 0
			gpu_objsize = 0
			gpu_bgmap = 0
			gpu_bgtile = 1
			gpu_switchwin = 0
			gpu_winmap = 0
			gpu_switchlcd = 0
			If (val And $01) > 0 Then gpu_switchbg = 1
			If (val And $02) > 0 Then gpu_switchobj = 1
			If (val And $04) > 0 Then gpu_objsize = 1
			If (val And $08) > 0 Then gpu_bgmap = 1
			If (val And $10) > 0 Then gpu_bgtile = 0
			If (val And $20) > 0 Then gpu_switchwin = 1
			If (val And $40) > 0 Then gpu_winmap = 1
			If (val And $80) > 0 Then gpu_switchlcd = 1
		Else If adr = $FF41 Then ;STAT - LCDC Status
			gpu_ie_hblank = 0
			gpu_ie_vblank = 0
			gpu_ie_oam = 0
			gpu_ie_compare = 0
			If (val And $08) > 0 Then gpu_ie_hblank = 1
			If (val And $10) > 0 Then gpu_ie_vblank = 1
			If (val And $20) > 0 Then gpu_ie_oam = 1
			If (val And $40) > 0 Then gpu_ie_compare = 1
		Else If adr = $FF42 Then ;SCY - Scroll Y
			gpu_scy = val
		Else If adr = $FF43 Then ;SCX - Scroll X
			gpu_scx = val
		Else If adr = $FF45 Then ;LYC - LY Compare
			gpu_compare = val
		Else If adr = $FF46 Then ;DMA - DMA Transfer
			memory_dma(val)
		Else If adr = $FF47 Then ;BGP - Background Palette
			gpu_bgpal(0) = gpu_pal(val And $3)
			gpu_bgpal(1) = gpu_pal((val Shr 2) And $03)
			gpu_bgpal(2) = gpu_pal((val Shr 4) And $03)
			gpu_bgpal(3) = gpu_pal((val Shr 6) And $03)
		Else If adr = $FF48 Then ;OBP0 - Object Palette 0
			gpu_objpal0(0) = gpu_pal(val And $03)
			gpu_objpal0(1) = gpu_pal((val Shr 2) And $03)
			gpu_objpal0(2) = gpu_pal((val Shr 4) And $03)
			gpu_objpal0(3) = gpu_pal((val Shr 6) And $03)
		Else If adr = $FF49 Then ;OBP1 - Object Palette 1
			gpu_objpal1(0) = gpu_pal(val And $03)
			gpu_objpal1(1) = gpu_pal((val Shr 2) And $03)
			gpu_objpal1(2) = gpu_pal((val Shr 4) And $03)
			gpu_objpal1(3) = gpu_pal((val Shr 6) And $03)
		Else If adr = $FF4A Then ;WY - Window Y
			gpu_wy = val
		Else If adr = $FF4B Then ;WX - Window X
			gpu_wx = val
		End If
	Else If adr <= $FFFF ;Zero-Page Ram
		PokeByte (memory_zram, adr-$FF80, val)
		
		If adr = $FFFF Then ;IE - Interrupt Enable
			memory_ie = val
		End If		
	
	Else
		Notify "Unknown Memory Bank: " + Hex$(adr), 1
		End
	
	End If
End Function

Function memory_write_word(adr,val)
	memory_write_byte(adr,val And $FF)
	memory_write_byte(adr+1,(val Shr 8) And $FF)
End Function

Function memory_bank_controller(adr,val)
	Select memory_rom_type
		Case $00 ;ROM ONLY
		
		Case $01, $02, $03, $19 ;MBC1
			If adr <= $1FFF ;RAM Enable

			Else If adr <= $3FFF ;ROM Bank Number
				val = val And $1F
				If val = 0 Then val = 1
				memory_rom_bank = (memory_rom_bank And $60) Or val
				memory_rom_bank = memory_rom_bank And (memory_rom_banks-1)
				memory_rom_offset = ((memory_rom_bank-1) And (memory_rom_banks-1)) Shl 14
			Else If adr <= $5FFF ;RAM/ROM Upper Bits
				If memory_mode = 0 Then
					memory_rom_bank = ((val And $03) Shl 5) Or (memory_rom_bank And $1F)
					memory_rom_bank = memory_rom_bank And (memory_rom_banks-1)
					memory_rom_offset = ((memory_rom_bank-1) And (memory_rom_banks-1)) Shl 14
				End If
			Else If adr <= $7FFF ;ROM/RAM Mode Select
				memory_mode = val And $01
			End If
		Case $05, $06 ;MBC2
			Notify "MBC2 is not supported!", 1
			End
		Case $0F, $10, $11, $12, $13 ;MBC3
			Notify "MBC3 is not supported!", 1
			End
		Case $15, $16, $17 ;MBC4
			Notify "MBC4 is not supported!", 1
			End
		Case $19, $1A, $1B, $1C, $1D, $1E ;MBC5
			Notify "MBC5 is not supported!", 1
			End
		Default:
			Notify "Memory Bank Controller is not supported!", 1
			End
	End Select
End Function

Function memory_load_rom (file$)
	memory_rom_path$ = file$
	
	rom_stream = ReadFile (file$)
	
	If rom_stream = 0 Then
		Notify "File is not readable!", 1
		End
	EndIf
	
	memory_rom_size = FileSize (file$)
	memory_rom_banks = memory_rom_size Shr 14
	
	SeekFile (rom_stream, $0134)
	For n=0 To 15
		t = ReadByte (rom_stream)
		If t = 0 Then t = 32
		memory_rom_name$ = memory_rom_name$ + Chr$(t)
	Next
	
	SeekFile (rom_stream, $0147)
	memory_rom_type = ReadByte (rom_stream)
	
	SeekFile (rom_stream, $0000)
	memory_rom = CreateBank (memory_rom_size)
	ReadBytes (memory_rom, rom_stream, 0, memory_rom_size)

	CloseFile rom_stream
End Function

Function memory_dma(val)
	adr = val Shl 8
	
	For n=0 To $9F
		memory_write_byte($FE00+n,memory_read_byte(adr+n))
	Next
End Function